@@ -146,6 +146,98 @@ ISOLATION_LEVEL_REPEATABLE_READ = 2
 ISOLATION_LEVEL_SERIALIZABLE = 3
 ISOLATION_LEVEL_READ_COMMITED_READ_ONLY = 4
 
+# Database Parameter Block parameter
+isc_dpb_version1 = 1
+isc_dpb_version2 = 2
+isc_dpb_cdd_pathname = 1
+isc_dpb_allocation = 2
+isc_dpb_journal = 3
+isc_dpb_page_size = 4
+isc_dpb_num_buffers = 5
+isc_dpb_buffer_length = 6
+isc_dpb_debug = 7
+isc_dpb_garbage_collect = 8
+isc_dpb_verify = 9
+isc_dpb_sweep = 10
+isc_dpb_enable_journal = 11
+isc_dpb_disable_journal = 12
+isc_dpb_dbkey_scope = 13
+isc_dpb_number_of_users = 14
+isc_dpb_trace = 15
+isc_dpb_no_garbage_collect = 16
+isc_dpb_damaged = 17
+isc_dpb_license = 18
+isc_dpb_sys_user_name = 19
+isc_dpb_encrypt_key = 20
+isc_dpb_activate_shadow = 21
+isc_dpb_sweep_interval = 22
+isc_dpb_delete_shadow = 23
+isc_dpb_force_write = 24
+isc_dpb_begin_log = 25
+isc_dpb_quit_log = 26
+isc_dpb_no_reserve = 27
+isc_dpb_user_name = 28
+isc_dpb_password = 29
+isc_dpb_password_enc = 30
+isc_dpb_sys_user_name_enc = 31
+isc_dpb_interp = 32
+isc_dpb_online_dump = 33
+isc_dpb_old_file_size = 34
+isc_dpb_old_num_files = 35
+isc_dpb_old_file = 36
+isc_dpb_old_start_page = 37
+isc_dpb_old_start_seqno = 38
+isc_dpb_old_start_file = 39
+isc_dpb_drop_walfile = 40
+isc_dpb_old_dump_id = 41
+isc_dpb_wal_backup_dir = 42
+isc_dpb_wal_chkptlen = 43
+isc_dpb_wal_numbufs = 44
+isc_dpb_wal_bufsize = 45
+isc_dpb_wal_grp_cmt_wait = 46
+isc_dpb_lc_messages = 47
+isc_dpb_lc_ctype = 48
+isc_dpb_cache_manager = 49
+isc_dpb_shutdown = 50
+isc_dpb_online = 51
+isc_dpb_shutdown_delay = 52
+isc_dpb_reserved = 53
+isc_dpb_overwrite = 54
+isc_dpb_sec_attach = 55
+isc_dpb_disable_wal = 56
+isc_dpb_connect_timeout = 57
+isc_dpb_dummy_packet_interval = 58
+isc_dpb_gbak_attach = 59
+isc_dpb_sql_role_name = 60
+isc_dpb_set_page_buffers = 61
+isc_dpb_working_directory = 62
+isc_dpb_sql_dialect = 63
+isc_dpb_set_db_readonly = 64
+isc_dpb_set_db_sql_dialect = 65
+isc_dpb_gfix_attach = 66
+isc_dpb_gstat_attach = 67
+isc_dpb_set_db_charset = 68
+isc_dpb_gsec_attach = 69
+isc_dpb_address_path = 70
+isc_dpb_process_id = 71
+isc_dpb_no_db_triggers = 72
+isc_dpb_trusted_auth = 73
+isc_dpb_process_name = 74
+isc_dpb_trusted_role = 75
+isc_dpb_org_filename = 76
+isc_dpb_utf8_filename = 77
+isc_dpb_ext_call_depth = 78
+isc_dpb_auth_block = 79
+isc_dpb_client_version = 80
+isc_dpb_remote_protocol = 81
+isc_dpb_host_name = 82
+isc_dpb_os_user	 = 83
+isc_dpb_specific_auth_data = 84
+isc_dpb_auth_plugin_list = 85
+isc_dpb_auth_plugin_name = 86
+isc_dpb_config = 87
+
+# Transaction Parameter Block parameter
 isc_tpb_version1 = 1
 isc_tpb_version3 = 3
 isc_tpb_consistency = 1
@@ -338,3 +430,51 @@ isc_arg_netware = 16
 isc_arg_win32 = 17
 isc_arg_warning = 18
 isc_arg_sql_state = 19
+
+charset_map = {
+    # DB CHAR SET NAME    :   PYTHON CODEC NAME (CANONICAL)
+    # --------------------------------------------------------------------------
+    'OCTETS'              :   None, # Allow to pass through unchanged.
+    'UNICODE_FSS'         :   'utf_8',
+    'UTF8'                :   'utf_8', # (Firebird 2.0+)
+    'SJIS_0208'           :   'shift_jis',
+    'EUCJ_0208'           :   'euc_jp',
+    'DOS737'              :   'cp737',
+    'DOS437'              :   'cp437',
+    'DOS850'              :   'cp850',
+    'DOS865'              :   'cp865',
+    'DOS860'              :   'cp860',
+    'DOS863'              :   'cp863',
+    'DOS775'              :   'cp775',
+    'DOS862'              :   'cp862',
+    'DOS864'              :   'cp864',
+    'ISO8859_1'           :   'iso8859_1',
+    'ISO8859_2'           :   'iso8859_2',
+    'ISO8859_3'           :   'iso8859_3',
+    'ISO8859_4'           :   'iso8859_4',
+    'ISO8859_5'           :   'iso8859_5',
+    'ISO8859_6'           :   'iso8859_6',
+    'ISO8859_7'           :   'iso8859_7',
+    'ISO8859_8'           :   'iso8859_8',
+    'ISO8859_9'           :   'iso8859_9',
+    'ISO8859_13'          :   'iso8859_13',
+    'KSC_5601'            :   'euc_kr',
+    'DOS852'              :   'cp852',
+    'DOS857'              :   'cp857',
+    'DOS861'              :   'cp861',
+    'DOS866'              :   'cp866',
+    'DOS869'              :   'cp869',
+    'WIN1250'             :   'cp1250',
+    'WIN1251'             :   'cp1251',
+    'WIN1252'             :   'cp1252',
+    'WIN1253'             :   'cp1253',
+    'WIN1254'             :   'cp1254',
+    'BIG_5'               :   'big5',
+    'GB_2312'             :   'gb2312',
+    'WIN1255'             :   'cp1255',
+    'WIN1256'             :   'cp1256',
+    'WIN1257'             :   'cp1257',
+    'KOI8-R'              :   'koi8_r', # (Firebird 2.0+)
+    'KOI8-U'              :   'koi8_u', # (Firebird 2.0+)
+    'WIN1258'             :   'cp1258', # (Firebird 2.0+)
+}
