@@ -39,7 +39,7 @@ else:
         fcntl.fcntl(fd, fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)
 
 
-__version__ = '0.7.4'
+__version__ = '0.8.0'
 apilevel = '2.0'
 threadsafety = 1
 paramstyle = 'qmark'
@@ -194,15 +194,9 @@ class XSQLVAR:
 
     def value(self, raw_value):
         if self.sqltype == SQL_TYPE_TEXT:
-            if self.sqlsubtype == 1:      # OCTETS
-                return raw_value
-            else:
-                return self.bytes_to_str(raw_value)
+            return self.bytes_to_str(raw_value).rstrip()
         elif self.sqltype == SQL_TYPE_VARYING:
-            if self.sqlsubtype == 1:      # OCTETS
-                return raw_value
-            else:
-                return self.bytes_to_str(raw_value)
+            return self.bytes_to_str(raw_value)
         elif self.sqltype in (SQL_TYPE_SHORT, SQL_TYPE_LONG, SQL_TYPE_INT64):
             n = bytes_to_bint(raw_value)
             if self.sqlscale:
@@ -224,7 +218,7 @@ class XSQLVAR:
         elif self.sqltype == SQL_TYPE_DOUBLE:
             return struct.unpack('!d', raw_value)[0]
         elif self.sqltype == SQL_TYPE_BOOLEAN:
-            return True if raw_value[0] else False
+            return True if byte_to_int(raw_value[0]) != 0 else False
         else:
             return raw_value
 
@@ -275,6 +269,9 @@ def parse_select_items(buf, xsqlda, connection):
         if item == isc_info_sql_sqlda_seq:
             l = bytes_to_int(buf[i+1:i+3])
             index = bytes_to_int(buf[i+3:i+3+l])
+            xsqlda[index-1] = XSQLVAR(connection.bytes_to_ustr
+                                        if connection.use_unicode
+                                        else connection.bytes_to_str)
             xsqlda[index-1] = XSQLVAR(connection.bytes_to_str)
             i = i + 3 + l
         elif item == isc_info_sql_type:
@@ -362,6 +359,7 @@ class PreparedStatement:
         connection._op_allocate_statement()
         (h, oid, buf) = connection._op_response()
         self.stmt_handle = h
+        self.is_opened = False
 
         if explain_plan:
             connection._op_prepare_statement(
@@ -400,9 +398,7 @@ class PreparedStatement:
 class Cursor:
     def __init__(self, trans):
         self._transaction = trans
-        self.transaction.connection._op_allocate_statement()
-        (h, oid, buf) = self.transaction.connection._op_response()
-        self.stmt_handle = h
+        self.stmt_handle = None
         self.arraysize = 1
 
     def __enter__(self):
@@ -418,26 +414,39 @@ class Cursor:
     def _convert_params(self, params):
         cooked_params = []
         for param in params:
-            if type(param) == str:
+            if (type(param)==str or
+                (PYTHON_MAJOR_VER == 2 and type(param)==unicode)):
                 param = self.transaction.connection.str_to_bytes(param)
             cooked_params.append(param)
         return cooked_params
 
-    def _execute(self, query, params):
-        cooked_params = self._convert_params(params)
-
+    def _get_stmt_handle(self, query):
         if isinstance(query, PreparedStatement):
+            if query.is_opened:
+                self.transaction.connection._op_free_statement(
+                                            query.stmt_handle, 1) # DSQL_close
+                (h, oid, buf) = self.transaction.connection._op_response()
             stmt_handle = query.stmt_handle
             stmt_type = query.statement_type
             self._xsqlda = query._xsqlda
+            query.is_opened = True
         else:
-            stmt_handle = self.stmt_handle
+            if self.stmt_handle:
+                self.transaction.connection._op_free_statement(
+                                            self.stmt_handle, 2) # DSQL_drop
+                (h, oid, buf) = self.transaction.connection._op_response()
+            self.transaction.connection._op_allocate_statement()
+            (stmt_handle, oid, buf) = self.transaction.connection._op_response()
+            self.stmt_handle = stmt_handle
             self.transaction.connection._op_prepare_statement(stmt_handle,
                                         self.transaction.trans_handle, query)
             (h, oid, buf) = self.transaction.connection._op_response()
             stmt_type, self._xsqlda = parse_xsqlda(
                                 buf, self.transaction.connection, stmt_handle)
+        return stmt_type, stmt_handle
 
+    def _execute(self, stmt_handle, params):
+        cooked_params = self._convert_params(params)
         self.transaction.connection._op_execute(stmt_handle,
                                 self.transaction.trans_handle, cooked_params)
         try:
@@ -446,38 +455,34 @@ class Cursor:
             e = sys.exc_info()[1]
             if 335544665 in e.gds_codes:
                 raise IntegrityError(e._message, e.gds_codes, e.sql_code)
-        return stmt_type, stmt_handle
-
-    def _callproc(self, query, params):
-        cooked_params = self._convert_params(params)
-        stmt_handle = self.stmt_handle
-        self.transaction.connection._op_prepare_statement(stmt_handle,
-                                        self.transaction.trans_handle, query)
-        (h, oid, buf) = self.transaction.connection._op_response()
-        stmt_type, self._xsqlda = parse_xsqlda(
-                            buf, self.transaction.connection, stmt_handle)
-
-        self.transaction.connection._op_execute2(stmt_handle,
-            self.transaction.trans_handle, cooked_params,
-            calc_blr(self._xsqlda))
-        return self.transaction.connection._op_sql_response(self._xsqlda)
+            raise OperationalError(e._message, e.gds_codes, e.sql_code)
 
     def prep(self, query, explain_plan=False):
         prepared_statement = PreparedStatement(self, query,
                                                 explain_plan=explain_plan)
         return prepared_statement
 
-    def execute(self, query, params = []):
-        stmt_type, stmt_handle = self._execute(query, params)
-        if stmt_type == isc_info_sql_stmt_select:
-            self._fetch_records = self._fetch_generator(stmt_handle)
-        else:
+    def execute(self, query, params=[]):
+        stmt_type, stmt_handle = self._get_stmt_handle(query)
+        if stmt_type == isc_info_sql_stmt_exec_procedure:
+            cooked_params = self._convert_params(params)
+            self.transaction.connection._op_execute2(stmt_handle,
+                self.transaction.trans_handle, cooked_params,
+                calc_blr(self._xsqlda))
+            self._callproc_result = \
+                self.transaction.connection._op_sql_response(self._xsqlda)
+            self.transaction.connection._op_response()
             self._fetch_records = None
+        else:
+            self._execute(stmt_handle, params)
+            if stmt_type == isc_info_sql_stmt_select:
+                self._fetch_records = self._fetch_generator(stmt_handle)
+            else:
+                self._fetch_records = None
 
-    def callproc(self, procname, params = []):
+    def callproc(self, procname, params=[]):
         query = 'EXECUTE PROCEDURE ' + procname + ' ' + ','.join('?'*len(params))
-        self._callproc_result = self._callproc(query, params)
-        self._fetch_records = None
+        return self.execute(query, params)
 
     def executemany(self, query, seq_of_params):
         for params in seq_of_params:
@@ -511,15 +516,10 @@ class Cursor:
                         connection._op_close_blob(h)
                         (h, oid, buf) = connection._op_response()
                         r[i] = v
+                        if x.sqlsubtype == 1:    # TEXT
+                            r[i] = connection.bytes_to_str(r[i])
                 yield r
 
-        # recreate stmt_handle
-        connection._op_free_statement(stmt_handle, 2) # DSQL_drop
-        (h, oid, buf) = connection._op_response()
-        connection._op_allocate_statement()
-        (h, oid, buf) = connection._op_response()
-        self.stmt_handle = h
-
         raise StopIteration()
 
     def fetchone(self):
@@ -533,9 +533,9 @@ class Cursor:
         # select statement
         try:
             if PYTHON_MAJOR_VER == 3:
-                return next(self._fetch_records)
+                return tuple(next(self._fetch_records))
             else:
-                return self._fetch_records.next()
+                return tuple(self._fetch_records.next())
         except StopIteration:
             return None
 
@@ -555,12 +555,12 @@ class Cursor:
         # callproc or not select statement
         if not self._fetch_records:
             if self._callproc_result:
-                r = [self._callproc_result]
+                r = [tuple(self._callproc_result)]
                 self._callproc_result = None
                 return r
             return None
         # select statement
-        return list(self._fetch_records)
+        return [tuple(r) for r in self._fetch_records]
 
     def fetchmany(self, size=None):
         if not size:
@@ -597,11 +597,11 @@ class Cursor:
             r = self.fetchonemap()
 
     def close(self):
-        if not hasattr(self, "stmt_handle"):
+        if not self.stmt_handle:
             return
         self.transaction.connection._op_free_statement(self.stmt_handle, 2)   # DSQL_drop
         (h, oid, buf) = self.transaction.connection._op_response()
-        delattr(self, "stmt_handle")
+        self.stmt_handle = None
 
     def nextset(self):
         raise NotSupportedError()
@@ -650,12 +650,12 @@ class Cursor:
         return count
 
 class EventConduit(WireProtocol):
-    def __init__(self, conn, names):
+    def __init__(self, conn, names, timeout):
         self.connection = conn
         self.event_names = {}
         for name in names:
             self.event_names[name] = 0
-
+        self.timeout = timeout
         (h, port, family, ip_address) = self.connection._op_connect_request()
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.sock.connect((ip_address, port))
@@ -665,7 +665,7 @@ class EventConduit(WireProtocol):
         self.connection._op_que_events(self.event_names, 0, 0, self.event_id)
         (h, oid, buf) = self.connection._op_response()
 
-        (event_id, event_names) = self._wait_for_event()
+        (event_id, event_names) = self._wait_for_event(timeout=timeout)
         assert event_id == self.event_id   # treat only one event_id
         self.event_names.update(event_names)
 
@@ -704,41 +704,37 @@ class Connection(WireProtocol):
                 + [4] + [len(hostname)] + [ord(c) for c in hostname] + [6, 0])
 
     def cursor(self):
-        if self.main_transaction is None:
+        if self._transaction is None:
             self.begin()
-        c = Cursor(self.main_transaction)
-        return c
+        return Cursor(self._transaction)
 
     def begin(self):
         if not self.sock:
             raise InternalError
-        trans = Transaction(self)
-        trans.begin()
-
-    @property
-    def transactions(self):
-        return self._transactions
-
-    @property
-    def main_transaction(self):
-        if len(self._transactions):
-            return self._transactions[0]
-        return None
+        if self._transaction is None:
+            self._transaction = Transaction(self)
+        self._transaction.begin()
 
     def commit(self, retaining=False):
-        if self.main_transaction:
-            self.main_transaction.commit(retaining=retaining)
+        if self._transaction:
+            self._transaction.commit(retaining=retaining)
 
     def savepoint(self, name):
-        return self.main_transaction.savepoint(name)
+        return self._transaction.savepoint(name)
 
     def rollback(self, retaining=False, savepoint=None):
-        if self.main_transaction:
-            self.main_transaction.rollback(retaining=retaining, savepoint=savepoint)
+        if self._transaction:
+            self._transaction.rollback(retaining=retaining, savepoint=savepoint)
 
-    def __init__(self, dsn=None, user=None, password=None, host=None,
+    def execute_immediate(self, query):
+        self._op_exec_immediate(
+            self._transaction.trans_handle, query=query)
+        (h, oid, buf) = self._op_response()
+
+    def __init__(self, dsn=None, user=None, password=None, role=None, host=None,
                     database=None, charset=DEFAULT_CHARSET, port=3050,
-                    page_size=None, is_services = False, cloexec=False):
+                    page_size=None, is_services=False, cloexec=False,
+                    timeout=None, isolation_level=None, use_unicode=None):
         if dsn:
             i = dsn.find(':')
             if i < 0:
@@ -761,9 +757,19 @@ class Connection(WireProtocol):
         self.port = port
         self.user = user
         self.password = password
+        self.role = role
         self.charset = charset
-        self._transactions = []
-        self.isolation_level = ISOLATION_LEVEL_READ_COMMITED
+        self.timeout = float(timeout) if timeout is not None else None
+        self.page_size = page_size
+        self.is_services = is_services
+        if isolation_level is None:
+            self.isolation_level = ISOLATION_LEVEL_READ_COMMITED
+        else:
+            self.isolation_level = int(isolation_level)
+        self.use_unicode = use_unicode
+        self.last_event_id = 0
+
+        self._transaction = None
 
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         if cloexec:
@@ -771,10 +777,14 @@ class Connection(WireProtocol):
         self.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         self.sock.connect((self.hostname, self.port))
 
-        self.page_size = page_size
-        self.is_services = is_services
         self._op_connect()
-        self._op_accept()
+        try:
+            self._op_accept()
+        except OperationalError:
+            self.sock.close()
+            self.sock = None
+            e = sys.exc_info()[1]
+            raise e
         if self.page_size:                      # create database
             self._op_create(self.page_size)
         elif self.is_services:                  # service api
@@ -783,19 +793,15 @@ class Connection(WireProtocol):
             self._op_attach()
         (h, oid, buf) = self._op_response()
         self.db_handle = h
-        self.last_event_id = 0
 
     def __enter__(self):
         return self
 
     def __exit__(self, exc, value, traceback):
-        if exc:
-            self.rollback()
-        else:
-            self.commit()
+        self.close()
 
     def set_isolation_level(self, isolation_level):
-        self.isolation_level = isolation_level
+        self.isolation_level = int(isolation_level)
 
     def _db_info(self, info_requests):
         if info_requests[-1] == isc_info_end:
@@ -929,15 +935,15 @@ class Connection(WireProtocol):
             return results
 
     def trans_info(self, info_requests):
-        if self.main_transaction:
-            return self.main_transaction.trans_info(info_requests)
+        if self._transaction:
+            return self._transaction.trans_info(info_requests)
         return {}
 
     def close(self):
         if self.sock is None:
             return
-        for trans in self._transactions:
-            trans.close()
+        if self._transaction:
+            self._transaction.close()
         if self.is_services:
             self._op_service_detach()
         else:
@@ -945,16 +951,17 @@ class Connection(WireProtocol):
         (h, oid, buf) = self._op_response()
         self.sock.close()
         self.sock = None
+        self.db_handle = None
 
     def drop_database(self):
         self._op_drop_database()
         (h, oid, buf) = self._op_response()
         self.sock.close()
         self.sock = None
-        delattr(self, "db_handle")
+        self.db_handle = None
 
-    def event_conduit(self, event_names):
-        return EventConduit(self, event_names)
+    def event_conduit(self, event_names, timeout=None):
+        return EventConduit(self, event_names, timeout)
 
     def __del__(self):
         if self.sock:
@@ -963,45 +970,58 @@ class Connection(WireProtocol):
 class Transaction:
     def __init__(self, connection, tpb=None):
         self._connection = connection
+        self._trans_handle = None
 
-    def begin(self):
-        self.close()
+    def _begin(self):
+        self._close()
         self.connection._op_transaction(
                 transaction_parameter_block[self.connection.isolation_level])
         (h, oid, buf) = self.connection._op_response()
-        self.trans_handle = h
-        self.connection._transactions.append(self)
+        self._trans_handle = h
+
+    def _close(self):
+        if self._trans_handle:
+            self.connection._op_rollback(self.trans_handle)
+            (h, oid, buf) = self.connection._op_response()
+        self._trans_handle = None
+
+    def begin(self):
+        self._begin()
 
     def commit(self, retaining=False):
+        if self._trans_handle is None:
+            return
         if retaining:
-            self.connection._op_commit_retaining(self.trans_handle)
+            self.connection._op_commit_retaining(self._trans_handle)
             (h, oid, buf) = self.connection._op_response()
         else:
-            self.connection._op_commit(self.trans_handle)
+            self.connection._op_commit(self._trans_handle)
             (h, oid, buf) = self.connection._op_response()
-            delattr(self, "trans_handle")
-            self.connection._transactions.remove(self)
+            self._trans_handle = None
 
     def savepoint(self, name):
-        self.connection._op_execute_immediate(self.trans_handle,
-                        self.connection.db_handle, in_msg='SAVEPOINT '+name)
+        if self._trans_handle is None:
+            return
+        self.connection._op_exec_immediate(self._trans_handle,
+                        query='SAVEPOINT '+name)
         (h, oid, buf) = self.connection._op_response()
 
     def rollback(self, retaining=False, savepoint=None):
+        if self._trans_handle is None:
+            return
         if savepoint:
-            self.connection._op_execute_immediate(self.trans_handle,
-                        -1, in_msg='ROLLBACK TO '+savepoint)
+            self.connection._op_exec_immediate(self._trans_handle,
+                        query='ROLLBACK TO '+savepoint)
             (h, oid, buf) = self.connection._op_response()
             return
 
         if retaining:
-            self.connection._op_rollback_retaining(self.trans_handle)
+            self.connection._op_rollback_retaining(self._trans_handle)
             (h, oid, buf) = self.connection._op_response()
         else:
-            self.connection._op_rollback(self.trans_handle)
+            self.connection._op_rollback(self._trans_handle)
             (h, oid, buf) = self.connection._op_response()
-            delattr(self, "trans_handle")
-            self.connection._transactions.remove(self)
+            self._trans_handle = None
 
     def _trans_info(self, info_requests):
         if info_requests[-1] == isc_info_end:
@@ -1044,20 +1064,17 @@ class Transaction:
             return results
 
     def close(self):
-        if self.closed:
-            return
-        self.connection._op_rollback(self.trans_handle)
-        (h, oid, buf) = self.connection._op_response()
-        delattr(self, "trans_handle")
-        self.connection._transactions.remove(self)
+        self._close()
 
     @property
     def connection(self):
         return self._connection
 
     @property
-    def closed(self):
-        return not hasattr(self, "trans_handle")
+    def trans_handle(self):
+        if self._trans_handle is None:
+            self._begin()
+        return self._trans_handle
 
 
 class RowMapping(Mapping):
