@@ -6,6 +6,7 @@
 #
 # Python DB-API 2.0 module for Firebird. 
 ##############################################################################
+import os
 import xdrlib, time, datetime, decimal, struct, select
 from firebirdsql.fberrmsgs import messages
 from firebirdsql import (DisconnectByPeer,
@@ -114,75 +115,9 @@ def byte_to_int(b):
     else:
         return ord(b)
 
-def recv_channel(sock, nbytes, timeout=None, word_alignment=False):
-    if timeout and select.select([sock], [], [], timeout)[0] == []:
-        return None
-
-    n = nbytes
-    if word_alignment and (n % 4):
-        n += 4 - nbytes % 4  # 4 bytes word alignment
-    r = bytes([])
-    while n:
-        b = sock.recv(n)
-        r += b
-        n -= len(b)
-    return r[:nbytes]
-
 def send_channel(sock, b):
     sock.send(b)
 
-def params_to_blr(params):
-    "Convert parameter array to BLR and values format."
-    ln = len(params) * 2
-    blr = bytes([5, 2, 4, 0, ln & 255, ln >> 8])
-    values = bytes([])
-    for p in params:
-        t = type(p)
-        if ((PYTHON_MAJOR_VER == 2 and t == str) or
-            (PYTHON_MAJOR_VER == 3 and t == bytes)):
-            v = p
-            nbytes = len(v)
-            pad_length = ((4-nbytes) & 3)
-            v += bytes([0]) * pad_length
-            blr += bytes([14, nbytes & 255, nbytes >> 8])
-        elif t == int:
-            v = bint_to_bytes(p, 4)
-            blr += bytes([8, 0])    # blr_long
-        elif t == decimal.Decimal or t == float:
-            if t == float:
-                p = decimal.Decimal(str(p))
-            (sign, digits, exponent) = p.as_tuple()
-            v = 0
-            ln = len(digits)
-            for i in range(ln):
-                v += digits[i] * (10 ** (ln -i-1))
-            if sign:
-                v *= -1
-            v = bint_to_bytes(v, 8)
-            if exponent < 0:
-                exponent += 256
-            blr += bytes([16, exponent])
-        elif t == datetime.date:
-            v = convert_date(p)
-            blr += bytes([12])
-        elif t == datetime.time:
-            v = convert_time(p)
-            blr += bytes([13])
-        elif t == datetime.datetime:
-            v = convert_timestamp(p)
-            blr += bytes([35])
-        elif t == bool:
-            v = bytes([1]) if p else bytes([0])
-            blr += bytes([23])
-        elif p == None:
-            v = bytes([0]) * 8
-            blr += bytes([9, 0])
-        values += v
-        blr += bytes([7, 0])
-        values += bytes([0]) * 4 if p != None else bytes([0xff,0xff,0x34,0x8c])
-    blr += bytes([255, 76])    # [blr_end, blr_eoc]
-    return blr, values
-
 
 class WireProtocol:
     buffer_length = 1024
@@ -190,6 +125,8 @@ class WireProtocol:
     op_connect = 1
     op_exit = 2
     op_accept = 3
+    op_reject = 4
+    op_protocol = 5
     op_disconnect = 6
     op_response = 9
     op_attach = 19
@@ -213,7 +150,7 @@ class WireProtocol:
     op_create_blob2 = 57
     op_allocate_statement = 62
     op_execute = 63
-    op_execute_immediate = 64
+    op_exec_immediate = 64
     op_fetch = 65
     op_fetch_response = 66
     op_free_statement = 67
@@ -229,104 +166,82 @@ class WireProtocol:
     op_service_start = 85
     op_rollback_retaining = 86
 
-    charset_map = {
-    # DB CHAR SET NAME    :   PYTHON CODEC NAME (CANONICAL)
-    # --------------------------------------------------------------------------
-    'OCTETS'              :   None, # Allow to pass through unchanged.
-    'UNICODE_FSS'         :   'utf_8',
-    'UTF8'                :   'utf_8', # (Firebird 2.0+)
-    'SJIS_0208'           :   'shift_jis',
-    'EUCJ_0208'           :   'euc_jp',
-    'DOS737'              :   'cp737',
-    'DOS437'              :   'cp437',
-    'DOS850'              :   'cp850',
-    'DOS865'              :   'cp865',
-    'DOS860'              :   'cp860',
-    'DOS863'              :   'cp863',
-    'DOS775'              :   'cp775',
-    'DOS862'              :   'cp862',
-    'DOS864'              :   'cp864',
-    'ISO8859_1'           :   'iso8859_1',
-    'ISO8859_2'           :   'iso8859_2',
-    'ISO8859_3'           :   'iso8859_3',
-    'ISO8859_4'           :   'iso8859_4',
-    'ISO8859_5'           :   'iso8859_5',
-    'ISO8859_6'           :   'iso8859_6',
-    'ISO8859_7'           :   'iso8859_7',
-    'ISO8859_8'           :   'iso8859_8',
-    'ISO8859_9'           :   'iso8859_9',
-    'ISO8859_13'          :   'iso8859_13',
-    'KSC_5601'            :   'euc_kr',
-    'DOS852'              :   'cp852',
-    'DOS857'              :   'cp857',
-    'DOS861'              :   'cp861',
-    'DOS866'              :   'cp866',
-    'DOS869'              :   'cp869',
-    'WIN1250'             :   'cp1250',
-    'WIN1251'             :   'cp1251',
-    'WIN1252'             :   'cp1252',
-    'WIN1253'             :   'cp1253',
-    'WIN1254'             :   'cp1254',
-    'BIG_5'               :   'big5',
-    'GB_2312'             :   'gb2312',
-    'WIN1255'             :   'cp1255',
-    'WIN1256'             :   'cp1256',
-    'WIN1257'             :   'cp1257',
-    'KOI8-R'              :   'koi8_r', # (Firebird 2.0+)
-    'KOI8-U'              :   'koi8_u', # (Firebird 2.0+)
-    'WIN1258'             :   'cp1258', # (Firebird 2.0+)
-    }
+
+    def recv_channel(self, nbytes, word_alignment=False):
+        n = nbytes
+        if word_alignment and (n % 4):
+            n += 4 - nbytes % 4  # 4 bytes word alignment
+        r = bytes([])
+        while n:
+            if (os.name != 'java'
+                and self.timeout is not None
+                and select.select([self.sock], [], [], self.timeout)[0] == []):
+                break
+            b = self.sock.recv(n)
+            if not b:
+                break
+            r += b
+            n -= len(b)
+        if len(r) < nbytes:
+            raise OperationalError('Can not recv() packets', None, None)
+        return r[:nbytes]
 
     def str_to_bytes(self, s):
-        return s.encode(self.charset_map.get(self.charset, self.charset))
+        "convert str to bytes"
+        if (PYTHON_MAJOR_VER == 3 or
+                (PYTHON_MAJOR_VER == 2 and type(s)==unicode)):
+            return s.encode(charset_map.get(self.charset, self.charset))
+        return s
 
     def bytes_to_str(self, b):
         "convert bytes array to raw string"
         if PYTHON_MAJOR_VER == 3:
-            return b.decode(self.charset_map.get(self.charset, self.charset))
+            return b.decode(charset_map.get(self.charset, self.charset))
         return b
 
     def bytes_to_ustr(self, b):
         "convert bytes array to unicode string"
-        return b.decode(self.charset_map.get(self.charset, self.charset))
+        return b.decode(charset_map.get(self.charset, self.charset))
 
     def _parse_status_vector(self):
         sql_code = 0
         gds_codes = set()
         message = ''
-        n = bytes_to_bint(recv_channel(self.sock, 4))
+        n = bytes_to_bint(self.recv_channel(4))
         while n != isc_arg_end:
             if n == isc_arg_gds:
-                gds_code = bytes_to_bint(recv_channel(self.sock, 4))
+                gds_code = bytes_to_bint(self.recv_channel(4))
                 if gds_code:
                     gds_codes.add(gds_code)
                     message += messages.get(gds_code, '@1')
                     num_arg = 0
             elif n == isc_arg_number:
-                num = bytes_to_bint(recv_channel(self.sock, 4))
+                num = bytes_to_bint(self.recv_channel(4))
                 if gds_code == 335544436:
                     sql_code = num
                 num_arg += 1
                 message = message.replace('@' + str(num_arg), str(num))
-            elif n == isc_arg_string or n == isc_arg_interpreted:
-                nbytes = bytes_to_bint(recv_channel(self.sock, 4))
-                s = str(recv_channel(self.sock, nbytes, word_alignment=True))
+            elif (n == isc_arg_string or
+                    n == isc_arg_interpreted
+                    or n == isc_arg_sql_state):
+                nbytes = bytes_to_bint(self.recv_channel(4))
+                s = str(self.recv_channel(nbytes, word_alignment=True))
                 num_arg += 1
                 message = message.replace('@' + str(num_arg), s)
             elif n == isc_arg_sql_state:
-                nbytes = bytes_to_bint(recv_channel(self.sock, 4))
-                s = str(recv_channel(self.sock, nbytes, word_alignment=True))
-            n = bytes_to_bint(recv_channel(self.sock, 4))
+                nbytes = bytes_to_bint(self.recv_channel(4))
+                s = str(self.recv_channel(nbytes, word_alignment=True))
+            n = bytes_to_bint(self.recv_channel(4))
 
         return (gds_codes, sql_code, message)
 
 
     def _parse_op_response(self):
-        b = recv_channel(self.sock, 16)
+        b = self.recv_channel(16)
         h = bytes_to_bint(b[0:4])         # Object handle
         oid = b[4:12]                       # Object ID
         buf_len = bytes_to_bint(b[12:])   # buffer length
-        buf = recv_channel(self.sock, buf_len, word_alignment=True)
+        buf = self.recv_channel(buf_len, word_alignment=True)
 
         (gds_codes, sql_code, message) = self._parse_status_vector()
         if sql_code or message:
@@ -335,13 +250,80 @@ class WireProtocol:
         return (h, oid, buf)
 
     def _parse_op_event(self):
-        b = recv_channel(self.sock, 4096) # too large TODO: read step by step
+        b = self.recv_channel(4096) # too large TODO: read step by step
         # TODO: parse event name
         db_handle = bytes_to_bint(b[0:4])
         event_id = bytes_to_bint(b[-4:])
 
         return (db_handle, event_id, {})
 
+    def params_to_blr(self, params):
+        "Convert parameter array to BLR and values format."
+        ln = len(params) * 2
+        blr = bytes([5, 2, 4, 0, ln & 255, ln >> 8])
+        values = bytes([])
+        for p in params:
+            t = type(p)
+            if ((PYTHON_MAJOR_VER == 2 and type(p) == unicode) or
+                (PYTHON_MAJOR_VER == 3 and type(p) == str)):
+                p = self.str_to_bytes(p)
+                t = type(p)
+            if ((PYTHON_MAJOR_VER == 2 and t == str) or
+                (PYTHON_MAJOR_VER == 3 and t == bytes)):
+                v = p
+                nbytes = len(v)
+                pad_length = ((4-nbytes) & 3)
+                v += bytes([0]) * pad_length
+                blr += bytes([14, nbytes & 255, nbytes >> 8])
+            elif t == int:
+                v = bint_to_bytes(p, 4)
+                blr += bytes([8, 0])    # blr_long
+            elif t == decimal.Decimal or t == float:
+                if t == float:
+                    p = decimal.Decimal(str(p))
+                (sign, digits, exponent) = p.as_tuple()
+                v = 0
+                ln = len(digits)
+                for i in range(ln):
+                    v += digits[i] * (10 ** (ln -i-1))
+                if sign:
+                    v *= -1
+                v = bint_to_bytes(v, 8)
+                if exponent < 0:
+                    exponent += 256
+                blr += bytes([16, exponent])
+            elif t == datetime.date:
+                v = convert_date(p)
+                blr += bytes([12])
+            elif t == datetime.time:
+                v = convert_time(p)
+                blr += bytes([13])
+            elif t == datetime.datetime:
+                v = convert_timestamp(p)
+                blr += bytes([35])
+            elif t == bool:
+                v = bytes([1, 0, 0, 0]) if p else bytes([0, 0, 0, 0])
+                blr += bytes([23])
+            else:   # fallback, convert to string
+                if p is None:
+                    v = bytes([])
+                else:
+                    p = p.__repr__()
+                    if (PYTHON_MAJOR_VER==3 or
+                        (PYTHON_MAJOR_VER == 2 and type(p)==unicode)):
+                        p = self.str_to_bytes(p)
+                    v = p
+                nbytes = len(v)
+                pad_length = ((4-nbytes) & 3)
+                v += bytes([0]) * pad_length
+                blr += bytes([14, nbytes & 255, nbytes >> 8])
+            values += v
+            blr += bytes([7, 0])
+            values += bytes([0]) * 4 if p != None else bytes([0xff,0xff,0xff,0xff])
+        blr += bytes([255, 76])    # [blr_end, blr_eoc]
+        return blr, values
+
+
     @wire_operation
     def _op_connect(self):
         p = xdrlib.Packer()
@@ -363,16 +345,19 @@ class WireProtocol:
     def _op_create(self, page_size=4096):
         dpb = bytes([1])
         s = self.str_to_bytes(self.charset)
-        dpb += bytes([68, len(s)]) + s
-        dpb += bytes([48, len(s)]) + s
+        dpb += bytes([isc_dpb_set_db_charset, len(s)]) + s
+        dpb += bytes([isc_dpb_lc_ctype, len(s)]) + s
         s = self.str_to_bytes(self.user)
-        dpb += bytes([28, len(s)]) + s
+        dpb += bytes([isc_dpb_user_name, len(s)]) + s
         s = self.str_to_bytes(self.password)
-        dpb += bytes([29, len(s)]) + s
-        dpb += bytes([63, 4]) + int_to_bytes(3, 4) # isc_dpb_sql_dialect = 3
-        dpb += bytes([24, 4]) + bint_to_bytes(1, 4) # isc_dpb_force_write = 1
-        dpb += bytes([54, 4]) + bint_to_bytes(1, 4) # isc_dpb_overwirte = 1
-        dpb += bytes([4, 4]) + int_to_bytes(page_size, 4)
+        dpb += bytes([isc_dpb_password, len(s)]) + s
+        if self.role:
+            s = self.str_to_bytes(self.role)
+            dpb += bytes([isc_dpb_sql_role_name, len(s)]) + s
+        dpb += bytes([isc_dpb_sql_dialect, 4]) + int_to_bytes(3, 4)
+        dpb += bytes([isc_dpb_force_write, 4]) + bint_to_bytes(1, 4)
+        dpb += bytes([isc_dpb_overwrite, 4]) + bint_to_bytes(1, 4)
+        dpb += bytes([isc_dpb_page_size, 4]) + int_to_bytes(page_size, 4)
         p = xdrlib.Packer()
         p.pack_int(self.op_create)
         p.pack_int(0)                       # Database Object ID
@@ -382,11 +367,13 @@ class WireProtocol:
 
     @wire_operation
     def _op_accept(self):
-        b = recv_channel(self.sock, 4)
+        b = self.recv_channel(4)
         while bytes_to_bint(b) == self.op_dummy:
-            b = recv_channel(self.sock, 4)
+            b = self.recv_channel(4)
+        if bytes_to_bint(b) == self.op_reject:
+            raise OperationalError('Connection is rejected', None, None)
         assert bytes_to_bint(b) == self.op_accept
-        b = recv_channel(self.sock, 12)
+        b = self.recv_channel(12)
         up = xdrlib.Unpacker(b)
         assert up.unpack_int() == 10
         assert  up.unpack_int() == 1
@@ -397,11 +384,14 @@ class WireProtocol:
     def _op_attach(self):
         dpb = bytes([1])
         s = self.str_to_bytes(self.charset)
-        dpb += bytes([48, len(s)]) + s
+        dpb += bytes([isc_dpb_lc_ctype, len(s)]) + s
         s = self.str_to_bytes(self.user)
-        dpb += bytes([28, len(s)]) + s
+        dpb += bytes([isc_dpb_user_name, len(s)]) + s
         s = self.str_to_bytes(self.password)
-        dpb += bytes([29, len(s)]) + s
+        dpb += bytes([isc_dpb_password, len(s)]) + s
+        if self.role:
+            s = self.str_to_bytes(self.role)
+            dpb += bytes([isc_dpb_sql_role_name, len(s)]) + s
         p = xdrlib.Packer()
         p.pack_int(self.op_attach)
         p.pack_int(0)                       # Database Object ID
@@ -566,7 +556,7 @@ class WireProtocol:
             p.pack_int(0)
             send_channel(self.sock, p.get_buffer())
         else:
-            (blr, values) = params_to_blr(params)
+            (blr, values) = self.params_to_blr(params)
             p.pack_bytes(blr)
             p.pack_int(0)
             p.pack_int(1)
@@ -585,7 +575,7 @@ class WireProtocol:
             p.pack_int(0)
             send_channel(self.sock, p.get_buffer())
         else:
-            (blr, values) = params_to_blr(params)
+            (blr, values) = self.params_to_blr(params)
             p.pack_bytes(blr)
             p.pack_int(0)
             p.pack_int(1)
@@ -597,27 +587,17 @@ class WireProtocol:
         send_channel(self.sock, p.get_buffer())
 
     @wire_operation
-    def _op_execute_immediate(self, trans_handle, db_handle, sql='', params=[],
-                                    in_msg='', out_msg='', possible_requests=0):
-        sql = self.str_to_bytes(sql)
-        in_msg = self.str_to_bytes(in_msg)
-        out_msg = self.str_to_bytes(out_msg)
-        r = bint_to_bytes(self.op_execute_immediate, 4)
-        r += bint_to_bytes(trans_handle, 4) + bint_to_bytes(db_handle, 4)
-        r += bint_to_bytes(len(sql), 2) + sql
-        r += bint_to_bytes(3, 2)    # dialect
-        if len(params) == 0:
-            r += bint_to_bytes(0, 2)    # in_blr len
-            values = bytes([])
-        else:
-            (blr, values) = params_to_blr(params)
-            r += bint_to_bytes(len(blr), 2) + blr
-        r += bint_to_bytes(len(in_msg), 2) + in_msg
-        r += bint_to_bytes(0, 2)    # unknown short int 0
-        r += bint_to_bytes(len(out_msg), 2) + out_msg
-        r += bint_to_bytes(possible_requests, 4)
-        r += bytes([0]) * ((4-len(r+values)) & 3)    # padding
-        send_channel(self.sock, r + values)
+    def _op_exec_immediate(self, trans_handle, query):
+        desc_items = bytes([])
+        p = xdrlib.Packer()
+        p.pack_int(self.op_exec_immediate)
+        p.pack_int(trans_handle)
+        p.pack_int(self.db_handle)
+        p.pack_int(3)   # dialect = 3
+        p.pack_string(self.str_to_bytes(query))
+        p.pack_bytes(desc_items)
+        p.pack_int(self.buffer_length)
+        send_channel(self.sock, p.get_buffer())
 
     @wire_operation
     def _op_fetch(self, stmt_handle, blr):
@@ -631,14 +611,14 @@ class WireProtocol:
 
     @wire_operation
     def _op_fetch_response(self, stmt_handle, xsqlda):
-        b = recv_channel(self.sock, 4)
+        b = self.recv_channel(4)
         while bytes_to_bint(b) == self.op_dummy:
-            b = recv_channel(self.sock, 4)
+            b = self.recv_channel(4)
         if bytes_to_bint(b) == self.op_response:
             return self._parse_op_response()    # error occured
         if bytes_to_bint(b) != self.op_fetch_response:
             raise InternalError
-        b = recv_channel(self.sock, 8)
+        b = self.recv_channel(8)
         status = bytes_to_bint(b[:4])
         count = bytes_to_bint(b[4:8])
         rows = []
@@ -647,15 +627,15 @@ class WireProtocol:
             for i in range(len(xsqlda)):
                 x = xsqlda[i]
                 if x.io_length() < 0:
-                    b = recv_channel(self.sock, 4)
+                    b = self.recv_channel(4)
                     ln = bytes_to_bint(b)
                 else:
                     ln = x.io_length()
-                raw_value = recv_channel(self.sock, ln, word_alignment=True)
-                if recv_channel(self.sock, 4) == bytes([0]) * 4: # Not NULL
+                raw_value = self.recv_channel(ln, word_alignment=True)
+                if self.recv_channel(4) == bytes([0]) * 4: # Not NULL
                     r[i] = x.value(raw_value)
             rows.append(r)
-            b = recv_channel(self.sock, 12)
+            b = self.recv_channel(12)
             op = bytes_to_bint(b[:4])
             status = bytes_to_bint(b[4:8])
             count = bytes_to_bint(b[8:])
@@ -746,22 +726,22 @@ class WireProtocol:
         p.pack_int(0)
         send_channel(self.sock, p.get_buffer())
 
-        b = recv_channel(self.sock, 4)
+        b = self.recv_channel(4)
         while bytes_to_bint(b) == self.op_dummy:
-            b = recv_channel(self.sock, 4)
+            b = self.recv_channel(4)
         if bytes_to_bint(b) != self.op_response:
             raise InternalError
 
-        h = bytes_to_bint(recv_channel(self.sock, 4))
-        recv_channel(self.sock, 8)  # garbase
-        ln = bytes_to_bint(recv_channel(self.sock, 4))
+        h = bytes_to_bint(self.recv_channel(4))
+        self.recv_channel(8)  # garbase
+        ln = bytes_to_bint(self.recv_channel(4))
         ln += ln % 4    # padding
-        family = bytes_to_bint(recv_channel(self.sock, 2))
-        port = bytes_to_bint(recv_channel(self.sock, 2), u=True)
-        b = recv_channel(self.sock, 4)
+        family = bytes_to_bint(self.recv_channel(2))
+        port = bytes_to_bint(self.recv_channel(2), u=True)
+        b = self.recv_channel(4)
         ip_address = '.'.join([str(byte_to_int(c)) for c in b])
         ln -= 8
-        recv_channel(self.sock, ln)
+        self.recv_channel(ln)
 
         (gds_codes, sql_code, message) = self._parse_status_vector()
         if sql_code or message:
@@ -771,18 +751,20 @@ class WireProtocol:
 
     @wire_operation
     def _op_response(self):
-        b = recv_channel(self.sock, 4)
+        b = self.recv_channel(4)
+        if b is None:
+            return
         while bytes_to_bint(b) == self.op_dummy:
-            b = recv_channel(self.sock, 4)
+            b = self.recv_channel(4)
         if bytes_to_bint(b) != self.op_response:
             raise InternalError
         return self._parse_op_response()
 
     @wire_operation
     def _op_event(self):
-        b = recv_channel(self.sock, 4)
+        b = self.recv_channel(4)
         while bytes_to_bint(b) == self.op_dummy:
-            b = recv_channel(self.sock, 4)
+            b = self.recv_channel(4)
         if bytes_to_bint(b) == self.op_response:
             return self._parse_op_response()
         if bytes_to_bint(b) == self.op_exit or bytes_to_bint(b) == self.op_exit:
@@ -793,38 +775,36 @@ class WireProtocol:
 
     @wire_operation
     def _op_sql_response(self, xsqlda):
-        b = recv_channel(self.sock, 4)
+        b = self.recv_channel(4)
         while bytes_to_bint(b) == self.op_dummy:
-            b = recv_channel(self.sock, 4)
+            b = self.recv_channel(4)
         if bytes_to_bint(b) != self.op_sql_response:
             raise InternalError
 
-        b = recv_channel(self.sock, 4)
+        b = self.recv_channel(4)
         count = bytes_to_bint(b[:4])
-
         r = []
+        if count == 0:
+            return []
         for i in range(len(xsqlda)):
             x = xsqlda[i]
             if x.io_length() < 0:
-                b = recv_channel(self.sock, 4)
+                b = self.recv_channel(4)
                 ln = bytes_to_bint(b)
             else:
                 ln = x.io_length()
-            raw_value = recv_channel(self.sock, ln, word_alignment=True)
-            if recv_channel(self.sock, 4) == bytes([0]) * 4: # Not NULL
+            raw_value = self.recv_channel(ln, word_alignment=True)
+            if self.recv_channel(4) == bytes([0]) * 4: # Not NULL
                 r.append(x.value(raw_value))
             else:
                 r.append(None)
-
-        b = recv_channel(self.sock, 32)     # ??? why 32 bytes skip
-
         return r
 
-    def _wait_for_event(self, timeout=None):
+    def _wait_for_event(self, timeout):
         event_names = {}
         event_id = 0
         while True:
-            b4 = recv_channel(self.sock, 4, timeout=timeout)
+            b4 = self.recv_channel(4)
             if b4 is None:
                 return None
             op = bytes_to_bint(b4)
@@ -833,9 +813,9 @@ class WireProtocol:
             elif op == self.op_exit or op == self.op_disconnect:
                 break
             elif op == self.op_event:
-                db_handle = bytes_to_int(recv_channel(self.sock, 4))
-                ln = bytes_to_bint(recv_channel(self.sock, 4))
-                b = recv_channel(self.sock, ln, word_alignment=True)
+                db_handle = bytes_to_int(self.recv_channel(4))
+                ln = bytes_to_bint(self.recv_channel(4))
+                b = self.recv_channel(ln, word_alignment=True)
                 assert byte_to_int(b[0]) == 1
                 i = 1
                 while i < len(b):
@@ -844,9 +824,9 @@ class WireProtocol:
                     n = bytes_to_int(b[i+1+ln:i+1+ln+4])
                     event_names[s] = n
                     i += ln + 5
-                recv_channel(self.sock, 8)  # ignore AST info
+                self.recv_channel(8)  # ignore AST info
 
-                event_id = bytes_to_bint(recv_channel(self.sock, 4))
+                event_id = bytes_to_bint(self.recv_channel(4))
                 break
             else:
                 raise InternalError
